local NAME, ADDON = ...

ADDON.cache = {}
ADDON.cache.__index = ADDON.cache

local cache = ADDON.cache
cache.items = {}
cache.bagContainers = {}

function cache:GetItem(bag, slot)
    self.items[bag] = self.items[bag] or {}
    self.items[bag][slot] = self.items[bag][slot] or ADDON.item(bag, slot)
    return self.items[bag][slot]
end

function cache:GetBagContainer()
    if not self.bagContainer then
        self.bagContainer = ADDON.categoryContainer('DJBagsBagCategoryContainer', UIParent)
        self.bagContainer:SetPoint('BOTTOMRIGHT', -200, 200)
        self.bagContainer:SetUserPlaced(true)
        self.bagContainer:Hide()
        self.bagContainer.moneyBar = ADDON.moneyBar(self.bagContainer)
        self.bagContainer.moneyBar:SetPoint('TOPRIGHT', self.bagContainer, 'BOTTOMRIGHT', 0, -2)
        self.bagContainer.moneyBar:Show()
    end
    return self.bagContainer
end

function cache:GetBagItemContainer(name)
    self.bagContainers[name] = self.bagContainers[name] or ADDON.itemContainer(name, name ~= ADDON.utils.EMPTY_BAG_NAME, nil, name == ADDON.utils.EMPTY_BAG_NAME)
    self:GetBagContainer():AddContainer(self.bagContainers[name])
    return self.bagContainers[name]
end

function cache:ShouldUpdate()
    return self:GetBagContainer():IsVisible()
end

function cache:UpdateBagArrangement()
    for _, container in pairs(self.bagContainers) do
        container:Arrange()
    end
    self:GetBagContainer():Arrange()
end

function cache:SetupSettings()
    local realm = GetRealmName()
    local player = UnitName("player")

    if DJBagsConfig and DJBagsConfig[realm] and DJBagsConfig[realm][player] then
        ADDON.settings = DJBagsConfig[realm][player]
    end
end

function cache:SettingsChanged()
    local realm = GetRealmName()
    local player = UnitName("player")
    if not DJBagsConfig then
        DJBagsConfig = {}
    end
    if not DJBagsConfig[realm] then
        DJBagsConfig[realm] = {}
    end
    DJBagsConfig[realm][player] = ADDON.settings

    self:GetBagContainer():Setup()
    self:GetBagContainer().moneyBar:Setup()

    for _, v in pairs(self.bagContainers) do
        v:Setup()
    end
    for _, slots in pairs(self.items) do
        for _, item in pairs(slots) do
            item:Setup()
        end
    end
end

function cache:GetContainerSorter(name)
    if name == 'D' then
       return function(A, B)
           if A.name == ADDON.utils.EMPTY_BAG_NAME then
               return true
           elseif B.name == ADDON.utils.EMPTY_BAG_NAME then
               return false
           else
               return A.name > B.name
           end
       end
    end
end

function cache:GetItemSorter(name)
    if name == 'D' then
       return function(A, B)
           if A.quality == B.quality then
               if A.ilevel == B.ilevel then
                   return A.name < B.name
               end
               return A.ilevel > B.ilevel
           end
           return A.quality > B.quality
       end
    end
end