local NAME, ADDON = ...

ADDON.settingsScreen = {}
ADDON.settingsScreen.__index = ADDON.settingsScreen

local settings = ADDON.settingsScreen

local function round(num)
    if num >= 0 then return math.floor(num+.5)
    else return math.ceil(num-.5) end
end

function settings:Init()
    ADDON.cache:SetupSettings()

    self.frame = ADDON.container('DJBagSettingFrame', UIParent)

    table.insert(UISpecialFrames, self.frame:GetName())
    self.frame:SetParent(UIParent)
    self.frame:SetMovable(true)
    self.frame:EnableMouse(true)
    self.frame:RegisterForDrag("LeftButton")
    self.frame:SetScript("OnDragStart", self.frame.StartMoving)
    self.frame:SetScript("OnDragStop", self.frame.StopMovingOrSizing)

    self.frame.title = self.frame:CreateFontString('DJBagSettingTitle', 'OVERLAY')
    self.frame.title:SetFont('Fonts\\FRIZQT__.TTF', 22, 'OUTLINE')
    self.frame.title:SetPoint('TOPLEFT', 5, -5)
    self.frame.title:SetText('DJBags Settings')

    self.frame.lastCnt = 0

    self:AddContainerSettings()
    self:AddItemSettings()
    self:AddItemContainerSettings()

    PanelTemplates_SetNumTabs(self.frame, self.frame.lastCnt);
    PanelTemplates_SetTab(self.frame, 1);

    self.frame:SetPoint('TOPLEFT', 200, -200)
    self.frame:SetSize(300, 65)
    self.frame:Hide()
end

function settings:AddContainerSettings()
    local frame = self:CreateSettingsFrame('Container')
    local btn = self:CreateTabButton('Container', frame)

    frame.borderColourSelector = self:CreateColorSelector(frame, 'Border Colour', function() return ADDON.settings.container.borderColor end, function(r, g, b, a)
        ADDON.settings.container.borderColor = {r, g, b, a}
        ADDON.cache:SettingsChanged()
    end)
    frame.borderColourSelector:SetPoint('TOPLEFT', 5, -5)

    frame.backgroundColourSelector = self:CreateColorSelector(frame, 'Background Colour', function() return ADDON.settings.container.backgroundColor end, function(r, g, b, a)
        ADDON.settings.container.backgroundColor = {r, g, b, a}
        ADDON.cache:SettingsChanged()
    end)
    frame.backgroundColourSelector:SetPoint('TOPLEFT', frame.borderColourSelector, "BOTTOMLEFT", 0, -5)

    frame.borderWidthSlider = self:CreateSlider(frame, 'Border Width', 1, 6, function ()
        return ADDON.settings.container.borderWidth
    end, function (width)
        ADDON.settings.container.borderWidth = width
        ADDON.cache:SettingsChanged()
    end)
    frame.borderWidthSlider:SetPoint('TOPLEFT', frame.backgroundColourSelector, "BOTTOMLEFT", 0, -20)

    frame:SetSize(160, 100)
    frame:Show()
    self.frame.visibleSettings = frame
end

function settings:AddItemSettings()
    local frame = self:CreateSettingsFrame('Items')
    local btn = self:CreateTabButton('Items', frame)

    frame.slider = self:CreateSlider(frame, 'Item Size', 25, 50, function()
        return ADDON.settings.item.size
    end, function(size)
        ADDON.settings.item.size = size
        ADDON.cache:SettingsChanged()
        ADDON.cache:UpdateBagArrangement()
    end)
    frame.slider:SetPoint('TOPLEFT', 5, -15)

    frame:SetSize(160, 45)
end

function settings:AddItemContainerSettings()
    local frame = self:CreateSettingsFrame('Items Container')
    local btn = self:CreateTabButton('Items Container', frame)

    frame.paddingSlider = self:CreateSlider(frame, 'Padding', 1, 15, function()
        return ADDON.settings.itemContainer.padding
    end, function(size)
        ADDON.settings.itemContainer.padding = size
        ADDON.cache:SettingsChanged()
        ADDON.cache:UpdateBagArrangement()
    end)
    frame.paddingSlider:SetPoint('TOPLEFT', 5, -15)

    frame.spacingSlider = self:CreateSlider(frame, 'Item Spacing', 1, 10, function()
        return ADDON.settings.itemContainer.spacing
    end, function(size)
        ADDON.settings.itemContainer.spacing = size
        ADDON.cache:SettingsChanged()
        ADDON.cache:UpdateBagArrangement()
    end)
    frame.spacingSlider:SetPoint('TOPLEFT', frame.paddingSlider, 'BOTTOMLEFT', 0, -20)

    frame.colSlider = self:CreateSlider(frame, 'Columns', 3, 10, function()
        return ADDON.settings.itemContainer.cols
    end, function(size)
        ADDON.settings.itemContainer.cols = size
        ADDON.cache:SettingsChanged()
        ADDON.cache:UpdateBagArrangement()
    end)
    frame.colSlider:SetPoint('TOPLEFT', frame.spacingSlider, 'BOTTOMLEFT', 0, -20)

    frame.fontSizeSlider = self:CreateSlider(frame, 'Font Size', 8, 16, function()
        return ADDON.settings.itemContainer.fontSize
    end, function(size)
        ADDON.settings.itemContainer.fontSize = size
        ADDON.cache:SettingsChanged()
        ADDON.cache:UpdateBagArrangement()
    end)
    frame.fontSizeSlider:SetPoint('TOPLEFT', frame.colSlider, 'BOTTOMLEFT', 0, -20)

    frame.fontMargingSlider = self:CreateSlider(frame, 'Title Margin', 1, 10, function()
        return ADDON.settings.itemContainer.titleMargin
    end, function(size)
        ADDON.settings.itemContainer.titleMargin = size
        ADDON.cache:SettingsChanged()
        ADDON.cache:UpdateBagArrangement()
    end)
    frame.fontMargingSlider:SetPoint('TOPLEFT', frame.fontSizeSlider, 'BOTTOMLEFT', 0, -20)


    frame.fontColorPicker = self:CreateColorSelector(frame, 'Title Color', function()
        return ADDON.settings.itemContainer.fontColor
    end, function(r, g, b, a)
        ADDON.settings.itemContainer.fontColor = {r, g, b, a}
        ADDON.cache:SettingsChanged()
    end)
    frame.fontColorPicker:SetPoint('TOPLEFT', frame.fontMargingSlider, 'BOTTOMLEFT', 0, -10)

    frame:SetSize(160, 220)
end

function settings:CreateSlider(parent, name, min, max, value, callBack)
    local slider = CreateFrame('Slider', name..'slider', parent, 'OptionsSliderTemplate')
    getglobal(slider:GetName().."Text"):SetText(name .. ' - ' .. tostring(value()));
    getglobal(slider:GetName().."High"):SetText(max);
    getglobal(slider:GetName().."Low"):SetText(min);
    slider:SetMinMaxValues(min, max)
    slider:SetValue(value())
    slider:Show()

    slider:SetScript('OnValueChanged', function (self, value)
        value = round(value)
        callBack(value)
        getglobal(self:GetName().."Text"):SetText(name .. ' - ' .. tostring(value));
    end)

    return slider
end

function settings:CreateColorSelector(parent, name, origColour, callBack)
    local colourSelector = CreateFrame('BUTTON', name..'colourSelector', parent)
    colourSelector:SetNormalFontObject("GameFontHighlight")
    colourSelector:SetText(name)
    colourSelector:SetBackdrop({
        bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
        edgeFile = "Interface\\Buttons\\WHITE8x8",
        tile = true, tileSize = 16, edgeSize = 1,
    })
    colourSelector:SetBackdropBorderColor(unpack(origColour()))
    colourSelector:SetBackdropColor(0, 0, 0, 0)
    colourSelector.call = callBack

    colourSelector:SetScript('OnClick', function(self)
        local r, g, b, a = unpack(origColour())

        local function callback(restore)
            local newR, newG, newB, newA = r, g, b, a;
            if not restore then
                newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
            end

            self:SetBackdropBorderColor(newR, newG, newB, newA)
            self.call(newR, newG, newB, newA)
        end
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = callback, callback, callback;
        ColorPickerFrame:SetColorRGB(r, g, b, a);
        ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a;
        ColorPickerFrame.previousValues = {r, g, b, a};
        ShowUIPanel(ColorPickerFrame)
    end)

    colourSelector:SetSize(150, 20)
    return colourSelector
end

function settings:CreateSettingsFrame(name)
    local frame = ADDON.container('DJSettingsFrame' .. name, self.frame)
    frame:SetPoint('TOP', self.frame, 'BOTTOM')
    frame:Hide()
    return frame
end

function settings:CreateTabButton(name, frame)
    local btn = CreateFrame('BUTTON', self.frame:GetName() .. 'Tab' .. (self.frame.lastCnt+1), self.frame, 'CharacterFrameTabButtonTemplate')
    btn:SetText(name)
    btn.cnt = (self.frame.lastCnt+1)
    btn:SetSize(75, 25)
    btn:SetScript('OnClick', function()
        PanelTemplates_SetTab(self.frame, btn.cnt);
        frame:Show()
        self.frame.visibleSettings:Hide()
        self.frame.visibleSettings = frame
    end)
    btn:Show()
    self:AddTabButton(btn)
    return btn
end

function settings:AddTabButton(button)
    button:SetPoint('TOPLEFT', self.frame.lastTab or self.frame,
        self.frame.lastTab and 'TOPRIGHT' or 'TOPLEFT', -8, self.frame.lastTab and 0 or -self.frame.title:GetStringHeight() - 5)
    self.frame.lastTab = button
    self.frame.lastCnt = self.frame.lastCnt + 1
end

function settings:Show()
    settings.frame:Show()
end